import { _base64Encode, _extend } from './utils';
export var parseFeatureFlagDecideResponse = function (response, persistence) {
    var flags = response['featureFlags'];
    if (flags) {
        // using the v1 api
        if (Array.isArray(flags)) {
            var $enabled_feature_flags = {};
            if (flags) {
                for (var i = 0; i < flags.length; i++) {
                    $enabled_feature_flags[flags[i]] = true;
                }
            }
            persistence &&
                persistence.register({
                    $active_feature_flags: flags,
                    $enabled_feature_flags: $enabled_feature_flags,
                });
        }
        else {
            // using the v2 api
            persistence &&
                persistence.register({
                    $active_feature_flags: Object.keys(flags || {}),
                    $enabled_feature_flags: flags || {},
                });
        }
    }
    else {
        if (persistence) {
            persistence.unregister('$active_feature_flags');
            persistence.unregister('$enabled_feature_flags');
        }
    }
};
var PostHogFeatureFlags = /** @class */ (function () {
    function PostHogFeatureFlags(instance) {
        this.instance = instance;
        this._override_warning = false;
        this.flagCallReported = {};
        this.featureFlagEventHandlers = [];
        this.reloadFeatureFlagsQueued = false;
        this.reloadFeatureFlagsInAction = false;
    }
    PostHogFeatureFlags.prototype.getFlags = function () {
        return Object.keys(this.getFlagVariants());
    };
    PostHogFeatureFlags.prototype.getFlagVariants = function () {
        var enabledFlags = this.instance.get_property('$enabled_feature_flags');
        var overriddenFlags = this.instance.get_property('$override_feature_flags');
        if (!overriddenFlags) {
            return enabledFlags || {};
        }
        var finalFlags = _extend({}, enabledFlags);
        var overriddenKeys = Object.keys(overriddenFlags);
        for (var i = 0; i < overriddenKeys.length; i++) {
            if (overriddenFlags[overriddenKeys[i]] === false) {
                delete finalFlags[overriddenKeys[i]];
            }
            else {
                finalFlags[overriddenKeys[i]] = overriddenFlags[overriddenKeys[i]];
            }
        }
        if (!this._override_warning) {
            console.warn('[PostHog] Overriding feature flags!', {
                enabledFlags: enabledFlags,
                overriddenFlags: overriddenFlags,
                finalFlags: finalFlags,
            });
            this._override_warning = true;
        }
        return finalFlags;
    };
    /**
     * Reloads feature flags asynchronously.
     *
     * Constraints:
     *
     * 1. Avoid parallel requests
     * 2. Delay a few milliseconds after each reloadFeatureFlags call to batch subsequent changes together
     * 3. Don't call this during initial load (as /decide will be called instead), see posthog-core.js
     */
    PostHogFeatureFlags.prototype.reloadFeatureFlags = function () {
        if (!this.reloadFeatureFlagsQueued) {
            this.reloadFeatureFlagsQueued = true;
            this._startReloadTimer();
        }
    };
    PostHogFeatureFlags.prototype.setAnonymousDistinctId = function (anon_distinct_id) {
        this.$anon_distinct_id = anon_distinct_id;
    };
    PostHogFeatureFlags.prototype.setReloadingPaused = function (isPaused) {
        this.reloadFeatureFlagsInAction = isPaused;
    };
    PostHogFeatureFlags.prototype.resetRequestQueue = function () {
        this.reloadFeatureFlagsQueued = false;
    };
    PostHogFeatureFlags.prototype._startReloadTimer = function () {
        var _this = this;
        if (this.reloadFeatureFlagsQueued && !this.reloadFeatureFlagsInAction) {
            setTimeout(function () {
                if (!_this.reloadFeatureFlagsInAction && _this.reloadFeatureFlagsQueued) {
                    _this.reloadFeatureFlagsQueued = false;
                    _this._reloadFeatureFlagsRequest();
                }
            }, 5);
        }
    };
    PostHogFeatureFlags.prototype._reloadFeatureFlagsRequest = function () {
        var _this = this;
        this.setReloadingPaused(true);
        var token = this.instance.get_config('token');
        var json_data = JSON.stringify({
            token: token,
            distinct_id: this.instance.get_distinct_id(),
            groups: this.instance.getGroups(),
            $anon_distinct_id: this.$anon_distinct_id,
        });
        var encoded_data = _base64Encode(json_data);
        this.instance._send_request(this.instance.get_config('api_host') + '/decide/?v=2', { data: encoded_data }, { method: 'POST' }, this.instance._prepare_callback(function (response) {
            // reset anon_distinct_id after at least a single request with it
            // makes it through
            _this.$anon_distinct_id = undefined;
            _this.receivedFeatureFlags(response);
            // :TRICKY: Reload - start another request if queued!
            _this.setReloadingPaused(false);
            _this._startReloadTimer();
        }));
    };
    /*
     * Get feature flag's value for user.
     *
     * ### Usage:
     *
     *     if(posthog.getFeatureFlag('my-flag') === 'some-variant') { // do something }
     *
     * @param {Object|String} key Key of the feature flag.
     * @param {Object|String} options (optional) If {send_event: false}, we won't send an $feature_flag_call event to PostHog.
     */
    PostHogFeatureFlags.prototype.getFeatureFlag = function (key, options) {
        if (options === void 0) { options = {}; }
        if (!this.getFlags()) {
            console.warn('getFeatureFlag for key "' + key + '" failed. Feature flags didn\'t load in time.');
            return false;
        }
        var flagValue = this.getFlagVariants()[key];
        if ((options.send_event || !('send_event' in options)) && !this.flagCallReported[key]) {
            this.flagCallReported[key] = true;
            this.instance.capture('$feature_flag_called', { $feature_flag: key, $feature_flag_response: flagValue });
        }
        return flagValue;
    };
    /*
     * See if feature flag is enabled for user.
     *
     * ### Usage:
     *
     *     if(posthog.isFeatureEnabled('beta-feature')) { // do something }
     *
     * @param {Object|String} key Key of the feature flag.
     * @param {Object|String} options (optional) If {send_event: false}, we won't send an $feature_flag_call event to PostHog.
     */
    PostHogFeatureFlags.prototype.isFeatureEnabled = function (key, options) {
        if (options === void 0) { options = {}; }
        if (!this.getFlags()) {
            console.warn('isFeatureEnabled for key "' + key + '" failed. Feature flags didn\'t load in time.');
            return false;
        }
        return !!this.getFeatureFlag(key, options);
    };
    PostHogFeatureFlags.prototype.addFeatureFlagsHandler = function (handler) {
        this.featureFlagEventHandlers.push(handler);
    };
    PostHogFeatureFlags.prototype.receivedFeatureFlags = function (response) {
        this.instance.decideEndpointWasHit = true;
        parseFeatureFlagDecideResponse(response, this.instance.persistence);
        var flags = this.getFlags();
        var variants = this.getFlagVariants();
        this.featureFlagEventHandlers.forEach(function (handler) { return handler(flags, variants); });
    };
    /*
     * Override feature flags for debugging.
     *
     * ### Usage:
     *
     *     - posthog.feature_flags.override(false)
     *     - posthog.feature_flags.override(['beta-feature'])
     *     - posthog.feature_flags.override({'beta-feature': 'variant', 'other-feature': True})
     *
     * @param {Object|Array|String} flags Flags to override with.
     */
    PostHogFeatureFlags.prototype.override = function (flags) {
        this._override_warning = false;
        if (flags === false) {
            this.instance.persistence.unregister('$override_feature_flags');
        }
        else if (Array.isArray(flags)) {
            var flagsObj = {};
            for (var i = 0; i < flags.length; i++) {
                flagsObj[flags[i]] = true;
            }
            this.instance.persistence.register({ $override_feature_flags: flagsObj });
        }
        else {
            this.instance.persistence.register({ $override_feature_flags: flags });
        }
    };
    /*
     * Register an event listener that runs when feature flags become available or when they change.
     * If there are flags, the listener is called immediately in addition to being called on future changes.
     *
     * ### Usage:
     *
     *     posthog.onFeatureFlags(function(featureFlags) { // do something })
     *
     * @param {Function} [callback] The callback function will be called once the feature flags are ready or when they are updated.
     *                              It'll return a list of feature flags enabled for the user.
     */
    PostHogFeatureFlags.prototype.onFeatureFlags = function (callback) {
        this.addFeatureFlagsHandler(callback);
        if (this.instance.decideEndpointWasHit) {
            var flags = this.getFlags();
            var flagVariants = this.getFlagVariants();
            callback(flags, flagVariants);
        }
    };
    return PostHogFeatureFlags;
}());
export { PostHogFeatureFlags };
//# sourceMappingURL=posthog-featureflags.js.map